@@ -11,45 +11,7 @@ from .statsig_user import StatsigUser
 from .client_initialize_formatter import ClientInitializeResponseFormatter
 from .evaluation_details import EvaluationDetails, EvaluationReason
 from .spec_store import _SpecStore
-
-
-class _ConfigEvaluation:
-
-    def __init__(self,
-                 fetch_from_server=False,
-                 boolean_value=False,
-                 json_value=None,
-                 rule_id="",
-                 secondary_exposures=None,
-                 allocated_experiment=None,
-                 explicit_parameters=None,
-                 is_experiment_group=False,
-                 evaluation_details=None,
-                 group_name=None):
-        if fetch_from_server is None:
-            fetch_from_server = False
-        self.fetch_from_server = fetch_from_server
-        if boolean_value is None:
-            boolean_value = False
-        self.boolean_value = boolean_value
-        if json_value is None:
-            json_value = {}
-        self.json_value = json_value
-        if rule_id is None:
-            rule_id = ""
-        self.rule_id = rule_id
-        if secondary_exposures is None:
-            secondary_exposures = []
-        if explicit_parameters is None:
-            explicit_parameters = []
-        self.secondary_exposures = secondary_exposures
-        self.undelegated_secondary_exposures = self.secondary_exposures
-        self.allocated_experiment = allocated_experiment
-        self.explicit_parameters = explicit_parameters
-        self.is_experiment_group = is_experiment_group is True
-        self.evaluation_details = evaluation_details
-        self.group_name = group_name
-
+from .config_evaluation import _ConfigEvaluation
 
 class _Evaluator:
     def __init__(self, spec_store: _SpecStore):
@@ -93,12 +55,12 @@ class _Evaluator:
         self._gate_overrides = {}
         self._config_overrides = {}
 
-    def get_client_initialize_response(self, user: StatsigUser):
+    def get_client_initialize_response(self, user: StatsigUser, client_sdk_key=None):
         if not self._spec_store.is_ready_for_checks():
             return None
 
         return ClientInitializeResponseFormatter \
-            .get_formatted_response(self.__eval_config, user, self._spec_store)
+            .get_formatted_response(self.__eval_config, user, self._spec_store, client_sdk_key)
 
     def _create_evaluation_details(self, reason: EvaluationReason):
         if reason == EvaluationReason.uninitialized:
@@ -207,8 +169,9 @@ class _Evaluator:
 
         for rule in config.get("rules", []):
             result = self.__evaluate_rule(user, rule)
-            if result.fetch_from_server:
-                return result
+            if result.unsupported:
+                return _ConfigEvaluation(True, False, default_value, "", exposures,
+                                        evaluation_details=self._create_evaluation_details(EvaluationReason.unsupported))
             if result.secondary_exposures is not None and len(
                     result.secondary_exposures) > 0:
                 exposures = exposures + result.secondary_exposures
@@ -238,7 +201,7 @@ class _Evaluator:
         eval_result = True
         for condition in rule.get("conditions", []):
             result = self.__evaluate_condition(user, condition)
-            if result.fetch_from_server:
+            if result.unsupported:
                 return result
             if result.secondary_exposures is not None and len(
                     result.secondary_exposures) > 0:
@@ -280,7 +243,7 @@ class _Evaluator:
             return _ConfigEvaluation(False, True)
         if type in ("FAIL_GATE", "PASS_GATE"):
             other_result = self.check_gate(user, target)
-            if other_result.fetch_from_server:
+            if other_result.unsupported:
                 return _ConfigEvaluation(True)
             new_exposure = {
                 "gate": target,
@@ -293,7 +256,30 @@ class _Evaluator:
                 exposures = other_result.secondary_exposures + exposures
             pass_gate = other_result.boolean_value if type == "PASS_GATE" else not other_result.boolean_value
             return _ConfigEvaluation(
-                other_result.fetch_from_server, pass_gate, {}, "", exposures)
+                False, pass_gate, {}, "", exposures)
+        if type in ("MULTI_PASS_GATE", "MULTI_FAIL_GATE"):
+            if target is None or len(target) == 0:
+                return _ConfigEvaluation(False, False)
+            pass_gate = False
+            exposures = []
+            for gate in target:
+                other_result = self.check_gate(user, gate)
+                if other_result.unsupported:
+                    return _ConfigEvaluation(True)
+                new_exposure = {
+                    "gate": gate,
+                    "gateValue": "true" if other_result.boolean_value else "false",
+                    "ruleID": other_result.rule_id
+                }
+                exposures.append(new_exposure)
+                if other_result.secondary_exposures is not None and len(
+                        other_result.secondary_exposures) > 0:
+                    exposures = exposures + other_result.secondary_exposures
+                pass_gate = pass_gate or other_result.boolean_value if type == "MULTI_PASS_GATE" else pass_gate or not other_result.boolean_value
+                if pass_gate:
+                    break
+            return _ConfigEvaluation(
+                False, pass_gate, {}, "", exposures)
         if type == "IP_BASED":
             value = self.__get_from_user(user, field)
             if value is None:
