@@ -9,8 +9,8 @@ from .statsig_errors import StatsigValueError, StatsigNameError
 from .statsig_network import _StatsigNetwork
 from .statsig_options import StatsigOptions
 from .thread_util import spawn_background_thread, THREAD_JOIN_TIMEOUT
-from .diagnostics import _Diagnostics
-from .utils import logger
+from .diagnostics import Context, Diagnostics
+from . import globals
 
 RULESETS_SYNC_INTERVAL = 10
 IDLISTS_SYNC_INTERVAL = 60
@@ -32,8 +32,7 @@ class _SpecStore:
     _background_download_id_lists: Optional[threading.Thread]
 
     def __init__(self, network: _StatsigNetwork, options: StatsigOptions, statsig_metadata: dict,
-                 error_boundary: _StatsigErrorBoundary, shutdown_event: threading.Event,
-                 init_diagnostics: _Diagnostics):
+                 error_boundary: _StatsigErrorBoundary, shutdown_event: threading.Event):
         self.last_update_time = 0
         self.initial_update_time = 0
         self.init_reason = EvaluationReason.uninitialized
@@ -47,23 +46,26 @@ class _SpecStore:
         self._background_download_configs = None
         self._background_download_id_lists = None
         self._sync_failure_count = 0
-        self._init_diagnostics = init_diagnostics
 
         self._configs = {}
         self._gates = {}
         self._layers = {}
         self._experiment_to_layer = {}
+        self._sdk_keys_to_app_ids = {}
 
         self._id_lists = {}
 
-        if not options.local_mode:
-            self._initialize_specs()
-            self.initial_update_time = -1 if self.last_update_time == 0 else self.last_update_time
+    def initialize(self):
+        if self._options.local_mode:
+            self._initialized = True
+            return
 
-            self.spawn_bg_threads_if_needed()
+        self._initialize_specs()
+        self.initial_update_time = -1 if self.last_update_time == 0 else self.last_update_time
 
-            self._download_id_lists()
+        self._download_id_lists(for_initialize=True)
 
+        self.spawn_bg_threads_if_needed()
         self._initialized = True
 
     def is_ready_for_checks(self):
@@ -72,6 +74,7 @@ class _SpecStore:
     def spawn_bg_threads_if_needed(self):
         if self._options.local_mode:
             return
+        Diagnostics.set_context(Context.CONFIG_SYNC.value)
 
         if self._background_download_configs is None or not self._background_download_configs.is_alive():
             self._spawn_bg_download_config_specs()
@@ -83,14 +86,14 @@ class _SpecStore:
         if self._options.local_mode:
             return
 
-        self._executor.shutdown(wait=False)
-
         if self._background_download_configs is not None:
             self._background_download_configs.join(THREAD_JOIN_TIMEOUT)
 
         if self._background_download_id_lists is not None:
             self._background_download_id_lists.join(THREAD_JOIN_TIMEOUT)
 
+        self._executor.shutdown(wait=False)
+
     def get_gate(self, name: str):
         return self._gates.get(name)
 
@@ -118,23 +121,27 @@ class _SpecStore:
     def get_all_id_lists(self):
         return self._id_lists
 
+    def get_target_app_for_sdk_key(self, sdk_key=None):
+        if sdk_key is None:
+            return None
+        return self._sdk_keys_to_app_ids.get(sdk_key)
+
     def _initialize_specs(self):
         if self._options.data_store is not None:
             if self._options.bootstrap_values is not None:
-                logger.warning(
+                globals.logger.debug(
                     "data_store gets priority over bootstrap_values. bootstrap_values will be ignored")
             self._load_config_specs_from_storage_adapter()
             if self.last_update_time == 0:
                 self._log_process("Retrying with network...")
-                self._download_config_specs()
+                self._download_config_specs(for_initialize=True)
 
         elif self._options.bootstrap_values is not None:
-            self._init_diagnostics.mark("bootstrap", "start", "load")
             self._bootstrap_config_specs()
-            self._init_diagnostics.mark("bootstrap", "end", "load")
 
-        else:
-            self._download_config_specs()
+        # If no updates from bootstrap or data_store, try to initialize from network
+        if self.init_reason is not EvaluationReason.bootstrap and self.last_update_time == 0:
+            self._download_config_specs(for_initialize=True)
 
     def _process_specs(self, specs_json) -> bool:
         self._log_process("Processing specs...")
@@ -161,11 +168,14 @@ class _SpecStore:
             for experiment_name in experiments:
                 new_experiment_to_layer[experiment_name] = layer_name
 
+        self._sdk_keys_to_app_ids = specs_json.get("sdk_keys_to_app_ids", {})
         self._gates = new_gates
         self._configs = new_configs
         self._layers = new_layers
         self._experiment_to_layer = new_experiment_to_layer
         self.last_update_time = specs_json.get("time", 0)
+        sampling_rate = specs_json.get("diagnostics", {})
+        Diagnostics.set_sampling_rate(sampling_rate)
 
         if callable(self._options.rules_updated_callback):
             self._options.rules_updated_callback(json.dumps(specs_json))
@@ -174,35 +184,42 @@ class _SpecStore:
         return True
 
     def _bootstrap_config_specs(self):
+        Diagnostics.mark().bootstrap().process().start()
         if self._options.bootstrap_values is None:
             return
 
         try:
             specs = json.loads(self._options.bootstrap_values)
-            if specs is None:
+            if specs is None or not _is_specs_json_valid(specs):
                 return
-
-            if not _is_specs_json_valid(specs):
-                return
-
             if self._process_specs(specs):
                 self.init_reason = EvaluationReason.bootstrap
+
         except ValueError:
             # JSON decoding failed, just let background thread update rulesets
-            logger.exception(
+            globals.logger.error(
                 'Failed to parse bootstrap_values')
-            return
+        finally:
+            Diagnostics.mark().bootstrap().process().end({'success': self.init_reason is EvaluationReason.bootstrap})
 
     def _spawn_bg_download_config_specs(self):
+        interval = self._options.rulesets_sync_interval or RULESETS_SYNC_INTERVAL
+        fast_start = self._sync_failure_count > 0
+
         self._background_download_configs = spawn_background_thread(
+            "bg_download_config_specs",
             self._sync,
-            (self._download_config_specs, self._options.
-             rulesets_sync_interval or RULESETS_SYNC_INTERVAL),
+            (self._download_config_specs, interval, fast_start),
             self._error_boundary)
 
-    def _download_config_specs(self):
+    def _download_config_specs(self, for_initialize=False):
         self._log_process("Loading specs from network...")
         log_on_exception = not self._initialized
+
+        timeout: Optional[int] = None
+        if for_initialize:
+            timeout = self._options.init_timeout
+
         if self._sync_failure_count * self._options.rulesets_sync_interval > 120:
             log_on_exception = True
             self._sync_failure_count = 0
@@ -211,24 +228,33 @@ class _SpecStore:
             specs = self._network.post_request("download_config_specs", {
                 "statsigMetadata": self._statsig_metadata,
                 "sinceTime": self.last_update_time,
-            }, log_on_exception, self._init_diagnostics)
+            }, log_on_exception, timeout)
+
+            if specs is None:
+                self._sync_failure_count += 1
+                return
+
+            self.download_config_spec_process(specs)
         except Exception as e:
             raise e
+        finally:
+            Diagnostics.log_diagnostics("config_sync", "download_config_specs")
 
-        if specs is None:
-            self._sync_failure_count += 1
-            return
+    def download_config_spec_process(self, specs):
+        try:
+            Diagnostics.mark().download_config_specs().process().start()
+            if not _is_specs_json_valid(specs):
+                return
 
-        self._init_diagnostics.mark("download_config_specs", "start", "process")
-        if not _is_specs_json_valid(specs):
-            self._init_diagnostics.mark("download_config_specs", "end", "process", False)
-            return
-
-        self._log_process("Done loading specs")
-        if self._process_specs(specs):
-            self._save_to_storage_adapter(specs)
-            self.init_reason = EvaluationReason.network
-        self._init_diagnostics.mark("download_config_specs", "end", "process", self.init_reason == EvaluationReason.network)
+            self._log_process("Done loading specs")
+            if self._process_specs(specs):
+                self._save_to_storage_adapter(specs)
+                self.init_reason = EvaluationReason.network
+        except Exception as e:
+            raise e
+        finally:
+            Diagnostics.mark().download_config_specs().process().end(
+                {'success': self.init_reason == EvaluationReason.network})
 
     def _save_to_storage_adapter(self, specs):
         if not _is_specs_json_valid(specs):
@@ -243,7 +269,6 @@ class _SpecStore:
         self._options.data_store.set(STORAGE_ADAPTER_KEY, json.dumps(specs))
 
     def _load_config_specs_from_storage_adapter(self):
-        self._init_diagnostics.mark("bootstrap", "start", "load")
         self._log_process("Loading specs from adapter")
         if self._options.data_store is None:
             return
@@ -254,7 +279,7 @@ class _SpecStore:
 
         cache = json.loads(cache_string)
         if not isinstance(cache, dict):
-            logger.warning(
+            globals.logger.warning(
                 "Invalid type returned from StatsigOptions.data_store")
             return
 
@@ -266,23 +291,38 @@ class _SpecStore:
         self._log_process("Done loading specs")
         if self._process_specs(cache):
             self.init_reason = EvaluationReason.data_adapter
-        self._init_diagnostics.mark("bootstrap", "end", "load")
 
     def _spawn_bg_download_id_lists(self):
-        self._background_download_id_lists = spawn_background_thread(self._sync, (
-            self._download_id_lists, self._options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL
-        ), self._error_boundary)
 
-    def _download_id_lists(self):
+        interval = self._options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL
+        self._background_download_id_lists = spawn_background_thread(
+            "bg_download_id_lists",
+            self._sync,
+            (self._download_id_lists, interval),
+            self._error_boundary)
+
+    def _download_id_lists(self, for_initialize=False):
         try:
+            timeout: Optional[int] = None
+            if for_initialize:
+                timeout = self._options.init_timeout
+
             server_id_lists = self._network.post_request("get_id_lists", {
                 "statsigMetadata": self._statsig_metadata,
-            }, False, self._init_diagnostics)
+            }, timeout=timeout)
 
             if server_id_lists is None:
                 return
-            self._init_diagnostics.mark("get_id_lists", "start", "process")
+            self._download_id_lists_process(server_id_lists)
+        except Exception as e:
+            raise e
+        finally:
+            Diagnostics.log_diagnostics("config_sync", "get_id_lists")
 
+    def _download_id_lists_process(self, server_id_lists):
+        threw_error = False
+        try:
+            Diagnostics.mark().get_id_list_sources().process().start({'idListCount': len(server_id_lists)})
             local_id_lists = self._id_lists
             workers = []
 
@@ -316,8 +356,7 @@ class _SpecStore:
                 if size <= read_bytes or url == "":
                     continue
 
-                # if the main id sync is killed, we should too
-                if not self._background_download_id_lists or not self._background_download_id_lists.is_alive():
+                if self._shutdown_event.is_set():
                     return
 
                 future = self._executor.submit(
@@ -336,19 +375,21 @@ class _SpecStore:
             # remove any list that has been deleted
             for list_name in deleted_lists:
                 local_id_lists.pop(list_name, None)
-            self._init_diagnostics.mark("get_id_lists", "end", "process", True)
 
         except Exception as e:
-            self._init_diagnostics.mark("get_id_lists", "end", "process", False)
-            self._error_boundary.log_exception(e)
-
+            threw_error = True
+            self._error_boundary.log_exception("_download_id_lists_process", e)
+        finally:
+            Diagnostics.mark().get_id_list_sources().process().end({'success': not threw_error})
     def _download_single_id_list(
             self, url, list_name, local_list, all_lists, start_index):
         resp = self._network.get_request(
             url, headers={"Range": f"bytes={start_index}-"})
         if resp is None:
             return
+        threw_error = False
         try:
+            Diagnostics.mark().get_id_list().process().start({'url': url})
             content_length_str = resp.headers.get('content-length')
             if content_length_str is None:
                 raise StatsigValueError("Content length invalid.")
@@ -372,18 +413,30 @@ class _SpecStore:
             local_list["readBytes"] = start_index + content_length
             all_lists[list_name] = local_list
         except Exception as e:
-            self._error_boundary.log_exception(e)
+            threw_error = True
+            self._error_boundary.log_exception("_download_single_id_list", e)
+        finally:
+            Diagnostics.mark().get_id_list().process().end({
+                'url': url,
+                'success': not threw_error,
+            })
+
+    def _sync(self, sync_func, interval, fast_start=False):
+        if fast_start:
+            sync_func()
 
-    def _sync(self, sync_func, interval):
         while True:
             try:
                 if self._shutdown_event.wait(interval):
                     break
                 sync_func()
             except Exception as e:
-                self._error_boundary.log_exception(e)
+                self._error_boundary.log_exception("_sync", e)
 
     def _log_process(self, msg, process=None):
         if process is None:
             process = "Initialize" if not self._initialized else "Sync"
-        logger.log_process(process, msg)
+        globals.logger.log_process(process, msg)
+
+    def _get_current_context(self):
+        return "initialize" if not self._initialized else "config_sync"
