@@ -1,8 +1,9 @@
 import traceback
 import requests
-from statsig.statsig_errors import StatsigNameError, StatsigRuntimeError, StatsigValueError
+from .statsig_errors import StatsigNameError, StatsigRuntimeError, StatsigValueError
 
-from statsig.statsig_metadata import _StatsigMetadata
+from .statsig_metadata import _StatsigMetadata
+from . import globals
 
 REQUEST_TIMEOUT = 20
 
@@ -19,27 +20,27 @@ class _StatsigErrorBoundary:
     def set_api_key(self, api_key):
         self._api_key = api_key
 
-    def capture(self, task, recover):
+    def capture(self, tag: str, task, recover):
         try:
             return task()
         except (StatsigValueError, StatsigNameError, StatsigRuntimeError) as e:
             raise e
         except Exception as e:
-            if self._is_silent is False:
-                print("[Statsig]: An unexpected error occurred.")
-                traceback.print_exc()
-
-            self.log_exception(e)
+            self.log_exception(tag, e)
             return recover()
 
-    def swallow(self, task):
+    def swallow(self, tag: str, task):
         def empty_recover():
             return None
 
-        self.capture(task, empty_recover)
+        self.capture(tag, task, empty_recover)
 
-    def log_exception(self, exception: Exception):
+    def log_exception(self, tag: str, exception: Exception, extra: dict = None):
         try:
+            if self._is_silent is False:
+                globals.logger.warning("[Statsig]: An unexpected error occurred.")
+                globals.logger.warning(traceback.format_exc())
+
             name = type(exception).__name__
             if self._api_key is None or name in self._seen:
                 return
@@ -49,7 +50,9 @@ class _StatsigErrorBoundary:
             requests.post(self.endpoint, json={
                 "exception": type(exception).__name__,
                 "info": traceback.format_exc(),
-                "statsigMetadata": _StatsigMetadata.get()
+                "statsigMetadata": _StatsigMetadata.get(),
+                "tag": tag,
+                "extra": extra
             }, headers={
                 'Content-type': 'application/json',
                 'STATSIG-API-KEY': self._api_key,
