@@ -1,16 +1,15 @@
 import collections
+import concurrent.futures
 import threading
-from typing import Optional, Union
 
+from typing import Optional, Union
 from .retryable_logs import RetryableLogs
 from .evaluation_details import EvaluationDetails
-from .evaluator import _ConfigEvaluation
+from .config_evaluation import _ConfigEvaluation
 from .statsig_event import StatsigEvent
-# from .statsig_user import StatsigUser
 from .layer import Layer
-from .utils import logger
+from . import globals
 from .thread_util import spawn_background_thread, THREAD_JOIN_TIMEOUT
-from .diagnostics import _Diagnostics
 
 _CONFIG_EXPOSURE_EVENT = "statsig::config_exposure"
 _LAYER_EXPOSURE_EVENT = "statsig::layer_exposure"
@@ -19,6 +18,7 @@ _DIAGNOSTICS_EVENT = "statsig::diagnostics"
 
 _IGNORED_METADATA_KEYS = {'serverTime', 'configSyncTime', 'initTime', 'reason'}
 
+
 def _safe_add_evaluation_to_event(
         evaluation_details: Union[EvaluationDetails, None], event: StatsigEvent):
     if evaluation_details is None or event is None or event.metadata is None:
@@ -42,7 +42,7 @@ class _StatsigLogger:
         self._net = net
         self._statsig_metadata = statsig_metadata
         self._local_mode = options.local_mode
-        self._console_logger = logger
+        self._console_logger = globals.logger
         self._logging_interval = options.logging_interval
         self._retry_interval = options.logging_interval
         self._event_queue_size = options.event_queue_size
@@ -52,6 +52,8 @@ class _StatsigLogger:
         self._background_retry = None
         self._background_deduper = None
         self.spawn_bg_threads_if_needed()
+        self._executor = concurrent.futures.ThreadPoolExecutor(max_workers=4)
+        self._futures = collections.deque(maxlen=10)
 
     def spawn_bg_threads_if_needed(self):
         if self._local_mode:
@@ -59,22 +61,22 @@ class _StatsigLogger:
 
         if self._background_flush is None or not self._background_flush.is_alive():
             self._background_flush = spawn_background_thread(
-                self._periodic_flush, (self._shutdown_event,), self._error_boundary)
+                "logger_background_flush", self._periodic_flush, (self._shutdown_event,), self._error_boundary)
 
         if self._background_retry is None or not self._background_retry.is_alive():
             self._background_retry = spawn_background_thread(
-                self._periodic_retry, (self._shutdown_event,), self._error_boundary)
+                "logger_background_retry", self._periodic_retry, (self._shutdown_event,), self._error_boundary)
 
         if self._background_deduper is None or not self._background_deduper.is_alive():
             self._background_deduper = spawn_background_thread(
-                self._periodic_dedupe_clear, (self._shutdown_event,), self._error_boundary)
+                "logger_background_deduper", self._periodic_dedupe_clear, (self._shutdown_event,), self._error_boundary)
 
     def log(self, event):
         if self._local_mode:
             return
         self._events.append(event.to_dict())
         if len(self._events) >= self._event_queue_size:
-            self.flush()
+            self.flush_in_background()
 
     def log_gate_exposure(self, user, gate, value, rule_id, secondary_exposures,
                           evaluation_details: EvaluationDetails, is_manual_exposure=False):
@@ -147,11 +149,15 @@ class _StatsigLogger:
 
         self.log(event)
 
-    def flush(self):
+    def flush_in_background(self):
         if len(self._events) == 0:
             return
         events_copy = self._events.copy()
         self._events = []
+
+        self._run_on_background_thread(lambda: self._flush_to_server(events_copy))
+
+    def _flush_to_server(self, events_copy):
         res = self._net.retryable_request("log_event", {
             "events": events_copy,
             "statsigMetadata": self._statsig_metadata,
@@ -159,6 +165,13 @@ class _StatsigLogger:
         if res is not None:
             self._retry_logs.append(RetryableLogs(res, 0))
 
+    def flush(self):
+        if len(self._events) == 0:
+            return
+        events_copy = self._events.copy()
+        self._events = []
+        self._flush_to_server(events_copy)
+
     def shutdown(self):
         self.flush()
 
@@ -168,14 +181,29 @@ class _StatsigLogger:
         if self._background_retry is not None:
             self._background_retry.join(THREAD_JOIN_TIMEOUT)
 
+        concurrent.futures.wait(self._futures, timeout=THREAD_JOIN_TIMEOUT)
+        self._futures.clear()
+        self._executor.shutdown()
+
+    def _run_on_background_thread(self, closure):
+        if self._shutdown_event.is_set():
+            return
+        future = self._executor.submit(closure)
+        self._futures.append(future)
+
+    def _flush_futures(self):
+        for future in concurrent.futures.as_completed(self._futures, timeout=THREAD_JOIN_TIMEOUT):
+            self._futures.remove(future)
+
     def _periodic_flush(self, shutdown_event):
         while True:
             try:
                 if shutdown_event.wait(self._logging_interval):
                     break
                 self.flush()
+                self._flush_futures()
             except Exception as e:
-                self._error_boundary.log_exception(e)
+                self._error_boundary.log_exception("_periodic_flush", e)
 
     def _periodic_dedupe_clear(self, shutdown_event):
         while True:
@@ -184,7 +212,7 @@ class _StatsigLogger:
                     break
                 self._deduper = set()
             except Exception as e:
-                self._error_boundary.log_exception(e)
+                self._error_boundary.log_exception("_periodic_dedupe_clear", e)
 
     def _periodic_retry(self, shutdown_event):
         while True:
@@ -207,10 +235,9 @@ class _StatsigLogger:
 
                     self._retry_logs.append(RetryableLogs(retry_logs.payload, retry_logs.retries))
 
-
-    def log_diagnostics_event(self, diagnostics: _Diagnostics):
+    def log_diagnostics_event(self, metadata):
         event = StatsigEvent(None, _DIAGNOSTICS_EVENT)
-        event.metadata = diagnostics.serialize()
+        event.metadata = metadata
         self.log(event)
 
     def _is_unique_exposure(self, user, eventName: str, metadata: dict or None) -> bool:
